"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomProtocolHandler = void 0;
const electron_1 = require("electron");
const path = require("path");
const url_1 = require("url");
const getLogger_1 = require("../logging/getLogger");
const EnvironmentUtil_1 = require("../runtime/EnvironmentUtil");
const config_1 = require("../settings/config");
const WindowManager_1 = require("../window/WindowManager");
const eventType_1 = require("./eventType");
const logger = getLogger_1.getLogger(path.basename(__filename));
const CORE_PROTOCOL_PREFIX = `${config_1.config.customProtocolName}://`;
const CORE_PROTOCOL_MAX_LENGTH = 1024;
var ProtocolCommand;
(function (ProtocolCommand) {
    ProtocolCommand["START_SSO_FLOW"] = "start-sso";
})(ProtocolCommand || (ProtocolCommand = {}));
class CustomProtocolHandler {
    constructor() {
        this.hashLocation = '';
        this.windowManager = WindowManager_1.WindowManager;
    }
    dispatchDeepLink(url) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.info(`Received deep link "${url}"`);
            if (typeof url === 'undefined' || !url.startsWith(CORE_PROTOCOL_PREFIX) || url.length > CORE_PROTOCOL_MAX_LENGTH) {
                logger.info('Invalid deep link, ignoring');
                return;
            }
            const route = new url_1.URL(url);
            if (route.host === ProtocolCommand.START_SSO_FLOW) {
                logger.info('Deep link is a SSO link, triggering SSO login ...');
                yield this.handleSSOLogin(route);
            }
            else {
                logger.info('Triggering hash location change ...');
                this.forwardHashLocation(route);
            }
        });
    }
    forwardHashLocation(route) {
        const location = route.href.substr(CORE_PROTOCOL_PREFIX.length);
        this.hashLocation = `/${location}`;
        logger.info(`New hash location: "${this.hashLocation}"`);
        this.windowManager.sendActionToPrimaryWindow(eventType_1.EVENT_TYPE.WEBAPP.CHANGE_LOCATION_HASH, this.hashLocation);
    }
    handleSSOLogin(route) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof route.pathname === 'string') {
                logger.info('Starting SSO flow ...');
                const code = route.pathname.trim().substr(1);
                try {
                    yield this.windowManager.sendActionAndFocusWindow(eventType_1.EVENT_TYPE.ACCOUNT.SSO_LOGIN, code);
                }
                catch (error) {
                    logger.error(`Cannot start SSO flow: ${error.message}`, error);
                }
            }
        });
    }
    findDeepLink(argv) {
        for (const arg of argv) {
            if (arg.startsWith(CORE_PROTOCOL_PREFIX)) {
                return arg;
            }
        }
    }
    registerCoreProtocol() {
        if (electron_1.app.isDefaultProtocolClient(config_1.config.customProtocolName)) {
            logger.info(`Custom protocol "${config_1.config.customProtocolName}" already registered`);
        }
        else {
            logger.info(`Registering custom protocol "${config_1.config.customProtocolName}" ...`);
            electron_1.app.setAsDefaultProtocolClient(config_1.config.customProtocolName);
        }
        if (EnvironmentUtil_1.platform.IS_MAC_OS) {
            electron_1.app.on('open-url', (event, url) => __awaiter(this, void 0, void 0, function* () {
                event.preventDefault();
                yield this.dispatchDeepLink(url);
            }));
        }
        else {
            electron_1.app.once('ready', () => __awaiter(this, void 0, void 0, function* () {
                logger.info('App ready, looking for deep link in arguments ...');
                const deepLink = this.findDeepLink(process.argv);
                if (deepLink) {
                    logger.info('App ready, dispatching deep link ...');
                    yield this.dispatchDeepLink(deepLink);
                }
            }));
            electron_1.app.on('second-instance', (_event, argv) => __awaiter(this, void 0, void 0, function* () {
                logger.info('Second instance detected, looking for deep link in arguments ...');
                const deepLink = this.findDeepLink(argv);
                if (deepLink) {
                    logger.info('Second instance detected, dispatching deep link ...');
                    yield this.dispatchDeepLink(deepLink);
                }
            }));
        }
    }
}
exports.CustomProtocolHandler = CustomProtocolHandler;
//# sourceMappingURL=CoreProtocol.js.map