"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TrayHandler = void 0;
const electron_1 = require("electron");
const path = require("path");
const locale = require("../locale/locale");
const EnvironmentUtil_1 = require("../runtime/EnvironmentUtil");
const lifecycle = require("../runtime/lifecycle");
const config_1 = require("../settings/config");
const WindowManager_1 = require("../window/WindowManager");
class TrayHandler {
    constructor() {
        this.lastUnreadCount = 0;
    }
    initTray(trayIcon = new electron_1.Tray(electron_1.nativeImage.createEmpty())) {
        const IMAGE_ROOT = path.join(electron_1.app.getAppPath(), config_1.config.electronDirectory, 'img');
        let trayPng = 'tray.png';
        let trayBadgePng = 'tray.badge.png';
        if (EnvironmentUtil_1.platform.IS_LINUX) {
            trayPng = `tray${EnvironmentUtil_1.linuxDesktop.isGnomeX11 ? '.gnome' : '@3x'}.png`;
            trayBadgePng = `tray.badge${EnvironmentUtil_1.linuxDesktop.isGnomeX11 ? '.gnome' : '@3x'}.png`;
        }
        const iconPaths = {
            badge: path.join(IMAGE_ROOT, 'taskbar.overlay.png'),
            tray: path.join(IMAGE_ROOT, 'tray-icon/tray', trayPng),
            trayWithBadge: path.join(IMAGE_ROOT, 'tray-icon/tray-with-badge', trayBadgePng),
        };
        this.icons = {
            badge: electron_1.nativeImage.createFromPath(iconPaths.badge),
            tray: electron_1.nativeImage.createFromPath(iconPaths.tray),
            trayWithBadge: electron_1.nativeImage.createFromPath(iconPaths.trayWithBadge),
        };
        this.trayIcon = trayIcon;
        this.trayIcon.setImage(this.icons.tray);
        this.buildTrayMenu();
    }
    showUnreadCount(win, count) {
        this.updateIcons(win, count);
        this.flashApplicationWindow(win, count);
        this.updateBadgeCount(count);
    }
    buildTrayMenu() {
        const contextMenu = electron_1.Menu.buildFromTemplate([
            {
                click: () => WindowManager_1.WindowManager.showPrimaryWindow(),
                label: locale.getText('trayOpen'),
            },
            {
                click: () => lifecycle.quit(),
                label: locale.getText('trayQuit'),
            },
        ]);
        if (this.trayIcon) {
            this.trayIcon.on('click', () => WindowManager_1.WindowManager.showPrimaryWindow());
            this.trayIcon.setContextMenu(contextMenu);
            this.trayIcon.setToolTip(config_1.config.name);
        }
    }
    flashApplicationWindow(win, count) {
        if (win.isFocused() || !count) {
            win.flashFrame(false);
        }
        else if (count > this.lastUnreadCount) {
            win.flashFrame(true);
        }
    }
    updateBadgeCount(count) {
        if (typeof count !== 'undefined') {
            electron_1.app.setBadgeCount(count);
            this.lastUnreadCount = count;
        }
    }
    updateIcons(win, count) {
        if (this.icons) {
            const trayImage = count ? this.icons.trayWithBadge : this.icons.tray;
            if (this.trayIcon) {
                this.trayIcon.setImage(trayImage);
            }
            const overlayImage = count ? this.icons.badge : null;
            win.setOverlayIcon(overlayImage, locale.getText('unreadMessages'));
        }
    }
}
exports.TrayHandler = TrayHandler;
//# sourceMappingURL=TrayHandler.js.map